--[[
The visual button for an instance of an action.  Based on settings, the button 
can look different.  It can look like a menu item, with the icon and spell name 
showing side by side, or look like a typical action button with just the icon 
showing.  Other appearances are possible as well.  This class controls 
implementing the standard appearance of the button, and how the visual effects 
actually appear (e.g., just the icon blinks, but the text does not, or the icon 
is highlighted, but the text is not).

An ActionButton instance is allocated for each action for a UnitGroupHeader.
Each ActionButton instance is associated with a single UnitGroupHeader and a 
single Action.

When an ActionButton is shown, it adjusts it's scale to the currently set scale,
then it tells the associated Action to identify the visual effects based on the 
state of the action and the unit the ActionButton is associated with.
--]]
uab.classes.ActionButton = uab.mixins.aceOO.Class()

function uab.classes.ActionButton.TimerEvent(self, event, elapsed)
    self[event](self, elapsed)
end

function uab.classes.ActionButton.prototype:init(header, action, number)
	uab.classes.ActionButton.super.prototype.init(self)
	
	self.header = header
    self.posstates = {}
    self.savedattrs = {}

    self.number = number
    self.framename = "uab_"..self.header.name.."_ActionBtn"..self.number
    self.timername = "ActionButtonTimer_"..self.framename
    self.maxalpha = 1.0

    --[[
	Create the action button frame
	--]]
    self.frame = getglobal(self.framename)
    if not self.frame then
        self.frame = CreateFrame("Button", self.framename, nil, "UAB_StandardActionButtonTemplate")
    	self.frame.itemcount = self.frame:CreateFontString(self.framename.."ItemCount", "ARTWORK", "NumberFontNormal")
    	self.frame.cooldown = self.frame:CreateFontString(self.framename.."Cooldown", "ARTWORK", "NumberFontNormal")
    	self.frame.border = self.frame:CreateTexture(self.framename.."Border", "OVERLAY")
    end

    self.frame.uabobj = self
    self.frame:Show()

    local count = self.frame.itemcount
    
    count:SetText("")
    count:SetPoint("BOTTOMRIGHT", self.frame, "BOTTOMRIGHT", -2, 2)
    local fontName, fontHeight, fontFlags = count:GetFont()
    count:SetFont(fontName, 10, fontFlags)

    local cooldown = self.frame.cooldown
    
    cooldown:SetText("")
    cooldown:SetPoint("TOP", self.frame, "TOP", 0, -2)
    fontName, fontHeight, fontFlags = cooldown:GetFont()
    cooldown:SetFont(fontName, 10, fontFlags)

    local border = self.frame.border
    
    border:SetTexture("Interface\\AddOns\\UnitActionBars\\images\\swborder")
    border:SetBlendMode("ADD")
    border:SetWidth(40)
    border:SetHeight(40)
    border:ClearAllPoints()
    border:SetPoint("CENTER", self.frame, "CENTER", 0, -1)
    border:Hide()
    
    uab.SetBoolFrameAttribute(self.frame, "useparent-unit", true)
    uab.SetBoolFrameAttribute(self.frame, "useparent-unitsuffix", true)

    self:Enable()
    self:SetAction(action)

    self:SetParent()
    self.frame:Hide()

    self.frame:SetScript("OnShow", function(self) self.uabobj:OnShow() end)
    self.frame:SetScript("OnHide", function(self) self.uabobj:OnHide() end)
    self.frame:SetScript("OnEnter", function(self, motion) self.uabobj:ShowTooltip() end)
    self.frame:SetScript("OnLeave", function(self, motion) GameTooltip:Hide() end)
end

function uab.classes.ActionButton.prototype:ClearAllPoints()
    for x, _ in pairs(self.posstates) do
        self.posstates[x].pt = nil
        self.posstates[x].x = nil
        self.posstates[x].y = nil
        self.posstates[x] = nil
    end
end

function uab.classes.ActionButton.prototype:SetPointForStates(states, ofspoint, ofsx, ofsy)
    self.posstates[states] = { pt = ofspoint, x = ofsx, y = ofsy }
end

function uab.classes.ActionButton.prototype:AllPointsSet()
    local ptattr, xattr, yattr
    
    for posstates, pos in pairs(self.posstates) do
        if not ptattr then
            ptattr = ""
            xattr = ""
            yattr = ""
        else
            ptattr = ptattr .. ";"
            xattr = xattr .. ";"
            yattr = yattr .. ";"
        end

        ptattr = ptattr .. posstates .. ":" .. pos.pt
        xattr = xattr .. posstates .. ":" .. pos.x
        yattr = yattr .. posstates .. ":" .. pos.y
    end

    if ptattr then
        uab.pdebug(self.frame:GetName().." (ofspoint) : "..ptattr)
        uab.pdebug(self.frame:GetName().." (ofsx) : "..xattr)
        uab.pdebug(self.frame:GetName().." (ofsy) : "..yattr)
    end

    uab.SetFrameAttribute(self.frame, "ofspoint", ptattr)
    uab.SetFrameAttribute(self.frame, "ofsx", xattr)
    uab.SetFrameAttribute(self.frame, "ofsy", yattr)
end

function uab.classes.ActionButton.prototype:SetParent()
    if self.header.frame then
        self.frame:SetParent(self.header.frame)
    	uab.SetFrameFrameAttribute(self.header.frame, "addchild", self.frame)
    end
end

function uab.classes.ActionButton.prototype:SetAction(action)
    if self.action then
        self.action:RemoveButton(self)
    end
    self.action = action
    self.actiondisabled = nil
    if self.action then
        self.action:AddButton(self)
    end
    
    self:CheckEnabledState()
end

function uab.classes.ActionButton.prototype:SetHostileAction(action)
    if self.hostileaction then
        self.hostileaction:RemoveButton(self)
    end
    self.hostileaction = action
    self.hostileactiondisabled = nil
    if self.hostileaction then
        self.hostileaction:AddButton(self)
    end
        
    self:CheckEnabledState()
end

function uab.classes.ActionButton.prototype:ClearAttributes(attributes)
	for name, _ in pairs(attributes) do
		if self.savedattrs[name] then
			uab.SetFrameAttribute(self.frame, name, self.savedattrs[name])
		else
			uab.SetFrameAttribute(self.frame, name, nil)
		end
	end
end

function uab.classes.ActionButton.prototype:SetAttributes(attributes)
	for name, val in pairs(attributes) do
		self.savedattrs[name] = self.frame:GetAttribute(name)
		uab.SetFrameAttribute(self.frame, name, val)
	end
end

function uab.classes.ActionButton.prototype:UnitStateModifiedAction()
    local unit = SecureButton_GetModifiedUnit(self:Frame())
    local action = self.action
    
    if UnitCanAttack("player", unit) and self.header.group.activator:HasHostileActions() then
        action = self.hostileaction
    end
    
    return action
end

function uab.classes.ActionButton.prototype:Show()
    local action = self:UnitStateModifiedAction()
             
    if not action or not action:GetData() then
        self.frame:SetAlpha(0)
    else
        self:StatusChanged(action)
    end
end

function uab.classes.ActionButton.prototype:Hide()
	self.frame:Hide()
end

function uab.classes.ActionButton.prototype:DisableAction()
    self.actiondisabled = true
    self:CheckEnabledState()
end

function uab.classes.ActionButton.prototype:DisableHostileAction()
    self.hostileactiondisabled = true
    self:CheckEnabledState()
end

function uab.classes.ActionButton.prototype:ActionDisabled()
    if self.actiondisabled then
        return true
    end
    
    if not self.action or not self.action:GetData() then
        return true
    end
end

function uab.classes.ActionButton.prototype:HostileActionDisabled()
    if self.hostileactiondisabled then
        return true
    end
    
    if not self.hostileaction or not self.hostileaction:GetData() then
        return true
    end
end

function uab.classes.ActionButton.prototype:Enable()
    self.enabled = true
    self:CheckEnabledState()
end

function uab.classes.ActionButton.prototype:Disable()
    self.enabled = nil
    self:CheckEnabledState()
end

function uab.classes.ActionButton.prototype:CheckEnabledState()
    if self.enabled and not (self:ActionDisabled() and self:HostileActionDisabled()) then
        self:DoEnable()
    else
        self:DoDisable()
    end
end

function uab.classes.ActionButton.prototype:DoEnable()
    if not self.isenabled then
        self.isenabled = true
        uab.pdebug("Enabling button: "..self.frame:GetName())
        uab.SetFrameAttribute(self.frame, "showstates", (self.header.states.offset + self.header.states.min).."-"..(self.header.states.offset + self.header.states.max))
        uab.SetFrameAttribute(self.frame, "hidestates", nil)
    
        uab.pdebug("Registering timer: "..self.timername)
        uab:RegisterMetro(
            self.timername,
            self.TimerEvent,
            0,
            self)
        self.timerregistered = true
        
        self:StayOpenChanged()
    end
end

function uab.classes.ActionButton.prototype:DoDisable()
    if self.isenabled then
        self.isenabled = nil
        
        uab.pdebug("Disabling button: "..self.frame:GetName())
        uab.SetFrameAttribute(self.frame, "showstates", nil)
        uab.SetFrameAttribute(self.frame, "hidestates", "*")
    
        uab.pdebug("Unregistering timer: "..self.timername)
        uab:UnregisterMetro(self.timername)
        self.timerregistered = nil
    
        self:StayOpenChanged()    
    end
end

function uab.classes.ActionButton.prototype:StayOpenChanged()
    if not self.header.group:IsStayOpen() or not self.isenabled then
        uab.SetFrameAttribute(self.frame, "newstate", "*:0")
    else
        uab.SetFrameAttribute(self.frame, "newstate", nil)
    end
end

function uab.classes.ActionButton.prototype:HighlightBorderChanged()
    local width = uab.db.profile.borderwidth
     
    if width > 5 then
        width = 5
    elseif width < 2 then
        width = 2
    end
    
    self.frame.border:SetTexture("Interface\\AddOns\\UnitActionBars\\images\\swborder" .. width)
end

function uab.classes.ActionButton.prototype:Frame()
	return self.frame
end

function uab.classes.ActionButton.prototype:BorderColor(r, g, b, a)
    if a == 0 then
        self.frame.border:Hide()
    else
        self.frame.border:SetVertexColor(r,g,b,a)
        self.frame.border:Show()
    end
end

function uab.classes.ActionButton.prototype:SetAlpha(alpha)
    self.maxalpha = alpha
end

function uab.classes.ActionButton.prototype:StartFlash()
    if not self.flashing then
        self.flashing = { timer = uab.db.profile.blinkcycle/2, direction = 1 }
    end
end

function uab.classes.ActionButton.prototype:EndFlash()
	if self.flashing then
        self.flashing = nil
    end
end

function uab.classes.ActionButton.prototype:StatusChanged(action)
    action:ShowButton(self)
end

function uab.classes.ActionButton.prototype:TimerEvent(elapsed)
    local a = self.maxalpha
    
    if self.flashing then
        self.flashing.timer = self.flashing.timer - elapsed
        if self.flashing.timer < 0 then
            self.flashing.timer = uab.db.profile.blinkcycle/2
            if self.flashing.direction then
                self.flashing.direction = nil
            else
                self.flashing.direction = 1
            end
        end
        
        if self.flashing.direction then
            a = ((self.flashing.timer * 2)/uab.db.profile.blinkcycle)*self.maxalpha
        else
            a = (1 - ((self.flashing.timer * 2)/uab.db.profile.blinkcycle))*self.maxalpha
        end
    end
    
    self.frame:SetAlpha(a)

    local cooldown

    local action = self:UnitStateModifiedAction()
    
    if not action or not action:GetData() then
        self.frame:SetAlpha(0)
    else
        cooldown = action:GetData():GetCooldown()

        if cooldown then
        	self.cooldown = cooldown
            self.frame.cooldown:SetText(cooldown)
            self.frame.cooldown:Show()
        elseif self.cooldown then
        	self.cooldown = nil
            self.frame.cooldown:SetText("")
            self.frame.cooldown:Hide()
        end
        
        self:StatusChanged(action)
    end
end

function uab.classes.ActionButton.prototype:IsUnit(unit)
	return unit == SecureButton_GetModifiedUnit(self.frame)
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.ActionButton.prototype:ShowTooltip()
    local action = self:UnitStateModifiedAction()
             
    if action and action:GetData() then
        action:ShowTooltip(self.frame)
    end
end

function uab.classes.ActionButton.prototype:OnShow()
    self.frame:SetFrameStrata("FULLSCREEN_DIALOG")
    if self.timerregistered then
        uab.pdebug("Starting timer: "..self.timername)
        uab:StartMetro(self.timername)
    end
end

function uab.classes.ActionButton.prototype:OnHide()
    if self.timerregistered then
        uab.pdebug("Stopping timer: "..self.timername)
        uab:StopMetro(self.timername)
    end
end

